<?php
    class DbManager {
		private $db;
		
		function DbManager() {
			try {
				$this->db = new PDO('sqlite:db/chatrooms.sqlite');
				$this->db->setAttribute(PDO::ATTR_PERSISTENT, true /*, PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION*/);
                $this->db->exec('PRAGMA temp_store = MEMORY; PRAGMA synchronous=OFF;');
			}
            catch (Exception $e) 
            {
				die('Error: database error.');
			}
		}
		
		function CreateChatroom($chatRoom) {
            if(is_null($chatRoom)) {
                die('Parameter error.');   
            }
            
			$query = 'INSERT INTO ChatRoom ( Id, DateCreation, DateLastNewMessage, ConnectedUsers, DateEnd, NoMoreThanOneVisitor, UserHash )';
			$query .= ' VALUES (?, ?, ?, ?, ?, ?, ?)';
			
			$req = $this->db->prepare($query);

			$req->execute(array($chatRoom->id, $chatRoom->dateCreation, $chatRoom->dateLastNewMessage, json_encode($chatRoom->users)
								, $chatRoom->dateEnd, $chatRoom->noMoreThanOneVisitor ? 1 : 0, $chatRoom->userId));
            if($this->db->errorCode() != '00000') 
            {
                die('Error: database error.');
            }
		}
        
        function CleanChatrooms($time) {
            $query = 'DELETE FROM ChatMessage WHERE IdChatRoom IN (SELECT Id FROM ChatRoom WHERE DateEnd < ?)';
            
            $req = $this->db->prepare($query);
            
            $req->execute(array($time));
            
            $query = 'DELETE FROM ChatRoom WHERE DateEnd < ?';
            
            $req = $this->db->prepare($query);
            
            $req->execute(array($time));
        }
        
        function DeleteChatroom($chatRoomId) {
            $query = 'DELETE FROM ChatMessage WHERE IdChatRoom = ?';
            
            $req = $this->db->prepare($query);
            
            $req->execute(array($chatRoomId));
            
            $query = 'DELETE FROM ChatRoom WHERE Id = ?';
            
            $req = $this->db->prepare($query);
            
            $req->execute(array($chatRoomId));
        }
        
        function GetChatroom($id) {
            if(is_null($id) || $id == '') {
                return null;    
            }
            
            $query = 'SELECT Id, DateCreation, DateLastNewMessage, ConnectedUsers, DateEnd, NoMoreThanOneVisitor, UserHash FROM ChatRoom WHERE Id = ?';
            
            $req = $this->db->prepare($query);
            
            $req->execute(array($id));
            
            $result = $req->fetchAll();
            
            if(is_null($result) || count($result) != 1) {
                return null;
            }
            
            $resultRow = $result[0];
            
            $chatRoom = new ChatRoom;
            $chatRoom->id = $resultRow['Id'];
            $chatRoom->dateCreation = $resultRow['DateCreation'];
            $chatRoom->dateLastNewMessage = $resultRow['DateLastNewMessage'];
            $chatRoom->noMoreThanOneVisitor = $resultRow['NoMoreThanOneVisitor'] == 1;
            $chatRoom->userId = $resultRow['UserHash'];
            $chatRoom->dateEnd = $resultRow['DateEnd'];
            $chatRoom->users = json_decode($resultRow['ConnectedUsers'], true);
            
            return $chatRoom;
        }
        
        function UpdateChatRoomUsers($chatRoom) {
            $query = 'UPDATE ChatRoom SET ConnectedUsers = ? WHERE Id = ?';
            
            $req = $this->db->prepare($query);
            
            $req->execute(array(json_encode($chatRoom->users), $chatRoom->id));
            
            if($this->db->errorCode() != '00000') 
            {
                die('Error: database error.');
            }
        }
        
        function GetLastMessages($chatRoomId, $nbMessages) {
            $query = 'SELECT Message, Hash, User, Date FROM ChatMessage WHERE IdChatRoom = ? ORDER BY Date DESC LIMIT ?';
            
            $req = $this->db->prepare($query);
            
            $req->execute(array($chatRoomId, $nbMessages));
            
            $messages = array();
            
            while ($line = $req->fetch()) { 
                $chatMessage = new ChatMessage;
                $chatMessage->message = $line['Message'];
                $chatMessage->hash = $line['Hash'];
                $chatMessage->userId = $line['User'];
                $chatMessage->date = $line['Date'];
                array_unshift($messages, $chatMessage);
            } 
            
            return $messages;
        }
        
        function UpdateChatRoomDateLastMessage($chatRoomId, $time) {
            $query = 'UPDATE ChatRoom SET DateLastNewMessage = ? WHERE Id = ?';
            
            $req = $this->db->prepare($query);
            
            $req->execute(array($time, $chatRoomId));
            
            if($this->db->errorCode() != '00000') 
            {
                die('Error: database error.');
            }
        }
        
        function AddMessage($chatRoomId, $message, $userMessage, $hash, $time) {
            $query = 'INSERT INTO ChatMessage ( IdChatRoom, Message, Hash, User, Date ) VALUES ( ?, ?, ?, ?, ? )';
            
            $req = $this->db->prepare($query);
            
            $req->execute(array($chatRoomId, $message, $hash, $userMessage, $time));
            
            if($this->db->errorCode() != '00000') 
            {
                die('Error: database error.');
            }
        }
        
        function GetNbChatRooms() {
            $query = 'SELECT COUNT(Id) FROM ChatRoom';
            
            $req = $this->db->prepare($query);
            
            $req->execute();
            
            $result = $req->fetchAll();
            
            if(is_null($result) || count($result) != 1) {
                return -1;
            }
            
            $resultRow = $result[0];
            
            return $resultRow[0];
        }
        
        function GetNbMessages() {
            $query = 'SELECT COUNT(IdChatRoom) FROM ChatMessage';
            
            $req = $this->db->prepare($query);
            
            $req->execute();
            
            $result = $req->fetchAll();
            
            if(is_null($result) || count($result) != 1) {
                return -1;
            }
            
            $resultRow = $result[0];
            
            return $resultRow[0];
        }
    }
?>